/*
 * Jednoduche kresleni v Jave.
 * 
 * Jiri Skala, brezen 2008
 * 
 */
package simpleDrawing;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import triangleRenderer.*;

/**
 * Jednoduche kresleni v Jave.
 * 
 * @author Jiri Skala
 */
public class SimpleDrawing extends JFrame {

	/**
	 * Panely pro usporadani ovladacich prvku v okne.
	 */
	private JPanel contentPane, panelTools;
	
	/**
	 * Panel kam budeme kreslit.
	 */
	private TriangleRenderer panelCanvas;
	
	/**
	 * Oteviraci dialog.
	 */
	private JFileChooser fileChooser;
	
	/**
	 * Tlacitko pro otevreni souboru.
	 */
	private JButton buttonLoad;
	
	/**
	 * Checkbox pro prepinani plny/drateny model.
	 */
	private JCheckBox checkBoxFill;
	
	
	
	
	
	/**
	 * Zobrazi okno aplikace.
	 * 
	 * @param args Command line arguments.
	 */
	public static void main(String[] args) {
		new SimpleDrawing().setVisible(true);
	}

	/**
	 * Konstruktor hlavniho okna.
	 */
	public SimpleDrawing() {
		
		// nastavime vlastnosti okna
		this.setTitle("Triangle Rendering");
		this.setSize(800, 600);
		this.setLocationRelativeTo(null);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		// poskladame GUI
		contentPane = new JPanel(new BorderLayout());
		this.setContentPane(contentPane);
		
		panelTools = new JPanel();
		//panelTools.setLayout(new BoxLayout(panelTools, BoxLayout.PAGE_AXIS));
		panelTools.setPreferredSize(new Dimension(200, 100));
		
		fileChooser = new JFileChooser();
		
		buttonLoad = new JButton("Load triangles...");
		buttonLoad.addActionListener(new ButtonActionListener());
		
		checkBoxFill = new JCheckBox("Fill triangles");
		checkBoxFill.addChangeListener(new CheckBoxChangeListener());
		checkBoxFill.setSelected(false);
		
		// pridame key listener vsem prvkum, ktere mohou mit focus
		KeyListener kl = new KeyListener();
		buttonLoad.addKeyListener(kl);
		checkBoxFill.addKeyListener(kl);
		
		panelTools.add(buttonLoad);
		panelTools.add(checkBoxFill);
		
		panelCanvas = new TriangleRenderer();
		
		contentPane.add(panelTools, BorderLayout.WEST);
		contentPane.add(panelCanvas, BorderLayout.CENTER);
	}
	
	private void move() {
		
		Vertex[] vOrig = panelCanvas.getOriginalVertices();
		Vertex[] vNew = new Vertex[vOrig.length];
		
		for (int i = 0; i < vOrig.length; i++)
			vNew[i] = new Vertex(vOrig[i].x+vOrig[i].y, vOrig[i].y, vOrig[i].z);
		
		panelCanvas.setTransformedVertices(vNew);
		panelCanvas.repaint();
	}
	
	
	
	/**
	 * Trida pro obsluhu udalosti tlacitka.
	 * 
	 * @author Jiri Skala
	 */
	private class ButtonActionListener implements ActionListener {
		
		public void actionPerformed(ActionEvent e) {
			
			int result = fileChooser.showOpenDialog(SimpleDrawing.this);
			if (result == JFileChooser.APPROVE_OPTION) {
				try {
					// otevreme soubor s trojuhelniky
					panelCanvas.loadTriangles(fileChooser.getSelectedFile());
					panelCanvas.repaint();
				}
				catch (java.io.IOException exc) {
					JOptionPane.showMessageDialog(SimpleDrawing.this, exc.getMessage(),
							"Error", JOptionPane.ERROR_MESSAGE);
				}
			}
		}
	}
	
	/**
	 * Trida pro obsluhu udalosti checkboxu.
	 * 
	 * @author Jiri Skala
	 */
	private class CheckBoxChangeListener implements ChangeListener {

		public void stateChanged(ChangeEvent e) {
			
			// nastavime vyplnovani/nevyplnovani podle stavu check boxu
			panelCanvas.setDoFill(checkBoxFill.isSelected());
			panelCanvas.repaint();
		}
	}
	
	/**
	 * Trida pro obsluhu udalosti klavesnice.
	 * 
	 * @author Jiri Skala
	 */
	private class KeyListener extends KeyAdapter {

		public void keyPressed(KeyEvent e) {
			
			switch (e.getKeyCode()) {
				case KeyEvent.VK_LEFT:
					move();
			}
			
			panelCanvas.repaint();
		}
	}
}
